/* This source file is part of the ATMEL AVR32-SoftwareFramework-AT32AP7000-1.0.0 Release */

/*This file is prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief SDRAMC on EBI driver for AVR32.
 *
 * - Compiler:           IAR EWAVR32 and GNU GCC for AVR32
 * - Supported devices:  All AVR32 devices with an SDRAMC module can be used.
 * - AppNote:
 *
 * \author               Atmel Corporation: http://www.atmel.com \n
 *                       Support and FAQ: http://support.atmel.no/
 *
 ******************************************************************************/

/* Copyright (C) 2006-2008, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "board.h"
#include "sdramc_at32ap7000.h"
#include "gpio.h"
#include "cycle_counter.h"
#include "pm_at32ap7000.h"

static void sdramc_enable_muxed_pins(void)
{
//	volatile avr32_hmatrix_t *hmatrix = &AVR32_HMATRIX;

	// Enable SDRAM mode for CS1 in the BAMBI mux
//	hmatrix->sfr[4] |= 0x0002;
//	hmatrix->sfr[4] |= 0x0100;

	static const gpio_map_t SDRAMC_EBI_GPIO_MAP =
	{
		// Enable DATA 16 through 31 pins, which is muxed with LCD
		{AVR32_EBI_DATA_16_PIN,           AVR32_EBI_DATA_16_FUNCTION          },
		{AVR32_EBI_DATA_17_PIN,           AVR32_EBI_DATA_17_FUNCTION          },
		{AVR32_EBI_DATA_18_PIN,           AVR32_EBI_DATA_18_FUNCTION          },
		{AVR32_EBI_DATA_19_PIN,           AVR32_EBI_DATA_19_FUNCTION          },
		{AVR32_EBI_DATA_20_PIN,           AVR32_EBI_DATA_20_FUNCTION          },
		{AVR32_EBI_DATA_21_PIN,           AVR32_EBI_DATA_21_FUNCTION          },
		{AVR32_EBI_DATA_22_PIN,           AVR32_EBI_DATA_22_FUNCTION          },
		{AVR32_EBI_DATA_23_PIN,           AVR32_EBI_DATA_23_FUNCTION          },
		{AVR32_EBI_DATA_24_PIN,           AVR32_EBI_DATA_24_FUNCTION          },
		{AVR32_EBI_DATA_25_PIN,           AVR32_EBI_DATA_25_FUNCTION          },
		{AVR32_EBI_DATA_26_PIN,           AVR32_EBI_DATA_26_FUNCTION          },
		{AVR32_EBI_DATA_27_PIN,           AVR32_EBI_DATA_27_FUNCTION          },
		{AVR32_EBI_DATA_28_PIN,           AVR32_EBI_DATA_28_FUNCTION          },
		{AVR32_EBI_DATA_29_PIN,           AVR32_EBI_DATA_29_FUNCTION          },
		{AVR32_EBI_DATA_30_PIN,           AVR32_EBI_DATA_30_FUNCTION          },
		{AVR32_EBI_DATA_31_PIN,           AVR32_EBI_DATA_31_FUNCTION          }
	};

	gpio_enable_module(SDRAMC_EBI_GPIO_MAP, sizeof(SDRAMC_EBI_GPIO_MAP) / sizeof(SDRAMC_EBI_GPIO_MAP[0]));
};

unsigned long sdramc_init( void *sdram_base, unsigned long hsb_hz )
{
	volatile avr32_hmatrix_t	*hmatrix	= &AVR32_HMATRIX;
	volatile avr32_sdramc_t		*sdramc		= &AVR32_SDRAMC;
//	volatile unsigned long		*sdram		= (void *) BOARD_SDRAM_BASE;
	volatile unsigned long		*sdram		= (void *) sdram_base;
	volatile unsigned long		read;
	unsigned long				i, size;


	if (SDRAM_DBW == 32)
	{
		sdramc_enable_muxed_pins();
	}

	hmatrix->sfr[AVR32_SDRAMC_HMATRIX_NR] |=  0x0002;							// Enable SDRAM mode for CS1 in BAMBI mux
//	hmatrix->sfr[AVR32_SDRAMC_HMATRIX_NR] |=  0x0100;
																				// Parameters for MT481C2M16B2TG SDRAM
	// Setup SDRAM controller
	sdramc->cr =	  ((SDRAM_ROW_BITS - 11)	<< AVR32_SDRAMC_CR_NR)			// number of rows  = 8192 (13 bits)
					| ((SDRAM_COL_BITS - 8)		<< AVR32_SDRAMC_CR_NC)			// number of cols  =  512 ( 9 bits)
					| ((SDRAM_BANK_BITS - 1)	<< AVR32_SDRAMC_CR_NB)			// number of banks =	4 ( 2 bits)
					| (SDRAM_CAS				<< AVR32_SDRAMC_CR_CAS)			// CAS latency	 =	3
					| (SDRAM_TWR				<< AVR32_SDRAMC_CR_TWR)			// write recovery time  = min 2 clk
					| (SDRAM_TRC				<< AVR32_SDRAMC_CR_TRC)			// active to active command period = min 55ns
					| (SDRAM_TRP				<< AVR32_SDRAMC_CR_TRP)			// precharge command period		= min 15ns
					| (SDRAM_TRCD				<< AVR32_SDRAMC_CR_TRCD)		// active to read or write delay   = min 15ns
					| (SDRAM_TRAS				<< AVR32_SDRAMC_CR_TRAS)		// active to precharge command	 = min 42ns
					| (SDRAM_TXSR				<< AVR32_SDRAMC_CR_TXSR)		// exit self refresh to active command = min 55ns
					| ((SDRAM_DBW == 16)		<< AVR32_SDRAMC_CR_DBW_OFFSET);	// data bus width for 16 bit

	sdramc->mr		=  AVR32_SDRAMC_MODE_NOP;									// issue NOP command to start the SDRAM clock
	read			=  sdramc->mr;												// dummy read
	sdram[0]		=  0;														// dummy write

	cpu_delay_us( 400L, pm_get_mclk_freq_hz() );

	sdramc->mr		=  AVR32_SDRAMC_MR_MODE_BANKS_PRECHARGE;					// issue PRECHARGE ALL command to SDRAM
	read			=  sdramc->mr;												// dummy read
	sdram[0]		=  0;

	sdramc->mr		=  AVR32_SDRAMC_MR_MODE_AUTO_REFRESH;						// issue eight AUTO-REFRESH cycles on SDRAM
	read			=  sdramc->mr;												// dummy read
	for (i = 0; i < 8; i++)
	{
		sdram[0]	=  0;
	}

	// A mode register set (MRS) cycle is issued to program
	// SDRAM parameters, in particular CAS latency and burst
	// length.

	// CAS from info struct, burst length 1, serial burst type
	sdramc->mr		=  AVR32_SDRAMC_MR_MODE_LOAD_MODE;							// issue mode register set cycle to program SDRAM
	read			=  sdramc->mr;												// dummy read
//	sdram[0x020]	=  0;
	sdram[0]		=  0;

																				// issue NORMAL command to SDRAM, 3 clk after tMRD
	read			=  sdramc->mr;												// dummy read
	sdramc->mr  	=  AVR32_SDRAMC_MR_MODE_NORMAL;
	read			=  sdramc->mr;												// dummy read
	sdram[0]		=  0;

//	hsbFreq = pm_read_module_freq_hz( PM_HSB_HEBI ) / 1000;
//	sdramc->tr	= (15625 * (hsb_hz / 1000)) / 1000000;							// write SDRAM refresh time for 15.625us
	sdramc->tr	= (781 * (hsb_hz / 1000)) / 100000;								// write SDRAM refresh time for 7.81 us

	if (SDRAM_DBW == 16)
	{
		size = 1 << (SDRAM_ROW_BITS + SDRAM_COL_BITS + SDRAM_BANK_BITS + 1);
	}
	else
	{
		size = 1 << (SDRAM_ROW_BITS + SDRAM_COL_BITS + SDRAM_BANK_BITS + 2);
	}

	return size;
}

